/** \file pollutionError_konvex.h */

/** \defgroup Estimator Estimator module
 * @{ <img src="estimator.png"> @}
 */

#ifndef AMDIS_POLLUTIONERROR_KONVEX_H
#define AMDIS_POLLUTIONERROR_KONVEX_H

#include "ResidualEstimator.h"
#include "PartitionElementData.h"

namespace AMDiS {

/** \ingroup Estimator
*	\brief Estimator for parallel solutions. 
*
* estimates the error using following formulae
* \f$||e_h||_{1,\Omega_0}=C_1 ||u_h||_{H_1(\Omega_1)}+\frac{C_2}{d}||u_h||_{L_2(\Omega\setminus\Omega_1}\f$
* where \f$\Omega\f$ is the complete area of interest and \f$\Omega_1\f$ the area of this process with overlapping. \f$\Omega_0\$ is the area of this process without overlapping, \f$d\$ 
* contains the distance dist(\f$\partial \Omega_0,\partial \Omega_1\f$)
*/
class PollutionErrorKonvex : public ResidualEstimator
{
   public:
   class Creator : public EstimatorCreator
    {
    public:
      Creator() : EstimatorCreator() {}

      virtual ~Creator() {}

      /// Returns a new ODirSolver object.
		Estimator* create() 
		{ 
			return new PollutionErrorKonvex(name, row);
		}
    };
    
    PollutionErrorKonvex(std::string name, int r);

	//sets the overlapping width for 2d-elements
    virtual void init(double timestep);

    virtual void estimateElement(ElInfo *elInfo);

    //virtual void exit(bool output=true);

    protected:
/**
  constant in front of the inner residual
*/
    double C1p;
/**
  constant in front of the outer residual
*/
    double C2p;

/**
  width of the overlapping area
*/
    double d;

	//traverse-class for edges and there normals in 2d
    class BoundaryTraverseStack : public TraverseStack
	{
		public:
			ElInfo* traverseFirst(Mesh *mesh, PartitionStatus p);
			ElInfo* traverseNext(ElInfo *old);
			
		private:
			PartitionStatus ps;
			bool proofConditions(ElInfo *elInfo);
	};
};
}
#endif